# Copyright 2015 Huawei Technologies Co., Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from mistralclient.commands.v2 import base


def format_list(service=None):
    columns = ('Name', 'Type')

    if service:
        data = (service.name, service.type)
    else:
        data = (tuple('' for _ in range(len(columns))),)

    return columns, data


class List(base.MistralLister):
    """List all services."""

    def _get_format_function(self):
        return format_list

    def _get_resources(self, parsed_args):
        mistral_client = self.app.client_manager.workflow_engine
        return mistral_client.services.list()



# Copyright 2016 - StackStorm, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import base64
import copy
from unittest import mock

from urllib import parse as urlparse

from oslo_utils import uuidutils
from osprofiler import _utils as osprofiler_utils
import osprofiler.profiler

from mistralclient.api import httpclient
from mistralclient.tests.unit import base

API_BASE_URL = 'http://localhost:8989/v2'
API_URL = '/executions'

EXPECTED_URL = API_BASE_URL + API_URL

AUTH_TOKEN = uuidutils.generate_uuid()
PROJECT_ID = uuidutils.generate_uuid()
USER_ID = uuidutils.generate_uuid()
REGION_NAME = 'fake_region'
PROFILER_HMAC_KEY = 'SECRET_HMAC_KEY'
PROFILER_TRACE_ID = uuidutils.generate_uuid()

EXPECTED_AUTH_HEADERS = {
    'X-Auth-Token': AUTH_TOKEN,
    'X-Project-Id': PROJECT_ID,
    'X-User-Id': USER_ID,
    'X-Region-Name': REGION_NAME
}

EXPECTED_REQ_OPTIONS = {
    'headers': EXPECTED_AUTH_HEADERS
}

EXPECTED_BODY = {
    'k1': 'abc',
    'k2': 123,
    'k3': True
}


class HTTPClientTest(base.BaseClientTest):

    def setUp(self):
        super(HTTPClientTest, self).setUp()
        self.client = httpclient.HTTPClient(
            API_BASE_URL,
            auth_token=AUTH_TOKEN,
            project_id=PROJECT_ID,
            user_id=USER_ID,
            region_name=REGION_NAME
        )

    def assertExpectedAuthHeaders(self):
        headers = self.requests_mock.last_request.headers

        self.assertEqual(AUTH_TOKEN, headers['X-Auth-Token'])
        self.assertEqual(PROJECT_ID, headers['X-Project-Id'])
        self.assertEqual(USER_ID, headers['X-User-Id'])

        return headers

    def assertExpectedBody(self):
        text = self.requests_mock.last_request.text
        form = urlparse.parse_qs(text, strict_parsing=True)

        self.assertEqual(len(EXPECTED_BODY), len(form))

        for k, v in EXPECTED_BODY.items():
            self.assertEqual([str(v)], form[k])

        return form

    def test_get_request_options(self):
        m = self.requests_mock.get(EXPECTED_URL, text='text')

        self.client.get(API_URL)

        self.assertTrue(m.called_once)
        self.assertExpectedAuthHeaders()

    @mock.patch.object(
        osprofiler.profiler._Profiler,
        'get_base_id',
        mock.MagicMock(return_value=PROFILER_TRACE_ID)
    )
    @mock.patch.object(
        osprofiler.profiler._Profiler,
        'get_id',
        mock.MagicMock(return_value=PROFILER_TRACE_ID)
    )
    def test_get_request_options_with_profile_enabled(self):
        m = self.requests_mock.get(EXPECTED_URL, text='text')
        osprofiler.profiler.init(PROFILER_HMAC_KEY)

        data = {'base_id': PROFILER_TRACE_ID, 'parent_id': PROFILER_TRACE_ID}
        signed_data = osprofiler_utils.signed_pack(data, PROFILER_HMAC_KEY)

        headers = {
            'X-Trace-Info': signed_data[0],
            'X-Trace-HMAC': signed_data[1]
        }

        self.client.get(API_URL)

        self.assertTrue(m.called_once)
        headers = self.assertExpectedAuthHeaders()
        self.assertEqual(signed_data[0], headers['X-Trace-Info'])
        self.assertEqual(signed_data[1], headers['X-Trace-HMAC'])

    def test_get_request_options_with_headers_for_get(self):
        m = self.requests_mock.get(EXPECTED_URL, text='text')
        target_auth_url = uuidutils.generate_uuid()
        target_auth_token = uuidutils.generate_uuid()
        target_user_id = 'target_user'
        target_project_id = 'target_project'
        target_service_catalog = 'this should be there'
        target_insecure = 'target insecure'
        target_region = 'target region name'
        target_user_domain_name = 'target user domain name'
        target_project_domain_name = 'target project domain name'

        target_client = httpclient.HTTPClient(
            API_BASE_URL,
            auth_token=AUTH_TOKEN,
            project_id=PROJECT_ID,
            user_id=USER_ID,
            region_name=REGION_NAME,
            target_auth_url=target_auth_url,
            target_auth_token=target_auth_token,
            target_project_id=target_project_id,
            target_user_id=target_user_id,
            target_service_catalog=target_service_catalog,
            target_region_name=target_region,
            target_user_domain_name=target_user_domain_name,
            target_project_domain_name=target_project_domain_name,
            target_insecure=target_insecure
        )

        target_client.get(API_URL)

        self.assertTrue(m.called_once)
        headers = self.assertExpectedAuthHeaders()
        self.assertEqual(target_auth_url, headers['X-Target-Auth-Uri'])
        self.assertEqual(target_auth_token, headers['X-Target-Auth-Token'])
        self.assertEqual(target_user_id, headers['X-Target-User-Id'])
        self.assertEqual(target_project_id, headers['X-Target-Project-Id'])
        self.assertEqual(str(target_insecure), headers['X-Target-Insecure'])
        self.assertEqual(target_region, headers['X-Target-Region-Name'])
        self.assertEqual(target_user_domain_name,
                         headers['X-Target-User-Domain-Name'])
        self.assertEqual(target_project_domain_name,
                         headers['X-Target-Project-Domain-Name'])

        catalog = base64.b64encode(target_service_catalog.encode('utf-8'))
        self.assertEqual(catalog, headers['X-Target-Service-Catalog'])

    def test_get_request_options_with_headers_for_post(self):
        m = self.requests_mock.post(EXPECTED_URL, text='text')
        headers = {'foo': 'bar'}

        self.client.post(API_URL, EXPECTED_BODY, headers=headers)

        self.assertTrue(m.called_once)
        headers = self.assertExpectedAuthHeaders()
        self.assertEqual('application/json', headers['Content-Type'])
        self.assertEqual('bar', headers['foo'])
        self.assertExpectedBody()

    def test_get_request_options_with_headers_for_put(self):
        m = self.requests_mock.put(EXPECTED_URL, text='text')
        headers = {'foo': 'bar'}

        self.client.put(API_URL, EXPECTED_BODY, headers=headers)

        self.assertTrue(m.called_once)
        headers = self.assertExpectedAuthHeaders()
        self.assertEqual('application/json', headers['Content-Type'])
        self.assertEqual('bar', headers['foo'])
        self.assertExpectedBody()

    def test_get_request_options_with_headers_for_delete(self):
        m = self.requests_mock.delete(EXPECTED_URL, text='text')
        headers = {'foo': 'bar'}

        self.client.delete(API_URL, headers=headers)

        self.assertTrue(m.called_once)
        headers = self.assertExpectedAuthHeaders()
        self.assertEqual('bar', headers['foo'])

    @mock.patch.object(
        httpclient.HTTPClient,
        '_get_request_options',
        mock.MagicMock(return_value=copy.deepcopy(EXPECTED_REQ_OPTIONS))
    )
    def test_http_get(self):
        m = self.requests_mock.get(EXPECTED_URL, text='text')
        self.client.get(API_URL)

        httpclient.HTTPClient._get_request_options.assert_called_with(
            'get',
            None
        )
        self.assertTrue(m.called_once)
        self.assertExpectedAuthHeaders()

    @mock.patch.object(
        httpclient.HTTPClient,
        '_get_request_options',
        mock.MagicMock(return_value=copy.deepcopy(EXPECTED_REQ_OPTIONS))
    )
    def test_http_post(self):
        m = self.requests_mock.post(EXPECTED_URL, status_code=201, text='text')
        self.client.post(API_URL, EXPECTED_BODY)

        httpclient.HTTPClient._get_request_options.assert_called_with(
            'post',
            None
        )

        self.assertTrue(m.called_once)
        self.assertExpectedAuthHeaders()
        self.assertExpectedBody()

    @mock.patch.object(
        httpclient.HTTPClient,
        '_get_request_options',
        mock.MagicMock(return_value=copy.deepcopy(EXPECTED_REQ_OPTIONS))
    )
    def test_http_put(self):
        m = self.requests_mock.put(EXPECTED_URL, json={})
        self.client.put(API_URL, EXPECTED_BODY)

        httpclient.HTTPClient._get_request_options.assert_called_with(
            'put',
            None
        )

        self.assertTrue(m.called_once)
        self.assertExpectedAuthHeaders()
        self.assertExpectedBody()

    @mock.patch.object(
        httpclient.HTTPClient,
        '_get_request_options',
        mock.MagicMock(return_value=copy.deepcopy(EXPECTED_REQ_OPTIONS))
    )
    def test_http_delete(self):
        m = self.requests_mock.delete(EXPECTED_URL, text='text')
        self.client.delete(API_URL)

        httpclient.HTTPClient._get_request_options.assert_called_with(
            'delete',
            None
        )

        self.assertTrue(m.called_once)
        self.assertExpectedAuthHeaders()



# Copyright 2020 Nokia Software.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from mistralclient.api import base


class CodeSource(base.Resource):
    resource_name = 'CodeSource'


class CodeSourceManager(base.ResourceManager):
    resource_class = CodeSource

    def create(self, name, content, namespace='', scope='private'):
        self._ensure_not_empty(name=name, content=content)

        # If the specified content is actually a file, read from it.
        content = self.get_contents_if_file(content)

        return self._create(
            '/code_sources?name=%s&scope=%s&namespace=%s' %
            (name, scope, namespace),
            content,
            dump_json=False,
            headers={'content-type': 'text/plain'}
        )

    def update(self, identifier, content, namespace='', scope='private'):
        self._ensure_not_empty(identifier=identifier, content=content)

        # If the specified content is actually a file, read from it.
        content = self.get_contents_if_file(content)

        return self._update(
            '/code_sources?identifier=%s&scope=%s&namespace=%s' %
            (identifier, scope, namespace),
            content,
            dump_json=False,
            headers={'content-type': 'text/plain'},
        )

    def list(self, namespace='', marker='', limit=None, sort_keys='',
             sort_dirs='', fields='', **filters):
        if namespace:
            filters['namespace'] = namespace

        query_string = self._build_query_params(
            marker=marker,
            limit=limit,
            sort_keys=sort_keys,
            sort_dirs=sort_dirs,
            fields=fields,
            filters=filters
        )

        return self._list(
            '/code_sources%s' % query_string,
            response_key='code_sources',
        )

    def get(self, identifier, namespace=''):
        self._ensure_not_empty(identifier=identifier)

        return self._get(
            '/code_sources/%s?namespace=%s' % (identifier, namespace)
        )

    def delete(self, identifier, namespace=None):
        self._ensure_not_empty(identifier=identifier)

        url = '/code_sources/%s' % identifier

        if namespace:
            url = url + '?namespace=%s' % namespace

        self._delete(url)



# Copyright 2020 Nokia Software.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from osc_lib.command import command

from mistralclient.commands.v2 import base
from mistralclient import utils


class DynamicActionFormatter(base.MistralFormatter):
    COLUMNS = [
        ('id', 'ID'),
        ('name', 'Name'),
        ('class_name', 'Class'),
        ('code_source_id', 'Code source ID'),
        ('code_source_name', 'Code source name'),
        ('project_id', 'Project ID'),
        ('scope', 'Scope'),
        ('created_at', 'Created at'),
        ('updated_at', 'Updated at'),
    ]

    @staticmethod
    def format(action=None, lister=False):
        if action:
            data = (
                action.id,
                action.name,
                action.class_name,
                action.code_source_id,
                action.code_source_name,
                action.project_id,
                action.scope,
                action.created_at
            )

            if hasattr(action, 'updated_at'):
                data += (action.updated_at,)
            else:
                data += (None,)

        else:
            data = (('',)*len(DynamicActionFormatter.COLUMNS),)

        return DynamicActionFormatter.headings(), data


class List(base.MistralLister):
    """List all dynamic actions."""

    def _get_format_function(self):
        return DynamicActionFormatter.format_list

    def get_parser(self, prog_name):
        parser = super(List, self).get_parser(prog_name)

        parser.add_argument(
            '--namespace',
            nargs='?',
            default='',
            help="Namespace of dynamic actions.",
        )

        return parser

    def _get_resources(self, parsed_args):
        mistral_client = self.app.client_manager.workflow_engine

        return mistral_client.dynamic_actions.list(
            marker=parsed_args.marker,
            limit=parsed_args.limit,
            sort_keys=parsed_args.sort_keys,
            sort_dirs=parsed_args.sort_dirs,
            fields=DynamicActionFormatter.fields(),
            namespace=parsed_args.namespace,
            **base.get_filters(parsed_args)
        )


class Get(command.ShowOne):
    """Show specific dynamic action."""

    def get_parser(self, prog_name):
        parser = super(Get, self).get_parser(prog_name)

        parser.add_argument(
            'identifier',
            help='Dynamic action identifier (name or ID)'
        )
        parser.add_argument(
            '--namespace',
            nargs='?',
            default='',
            help="Namespace to create the dynamic action within.",
        )

        return parser

    def take_action(self, parsed_args):
        mistral_client = self.app.client_manager.workflow_engine

        action = mistral_client.dynamic_actions.get(
            parsed_args.identifier,
            parsed_args.namespace
        )

        return DynamicActionFormatter.format(action)


class Create(command.ShowOne):
    """Create new action."""

    def get_parser(self, prog_name):
        parser = super(Create, self).get_parser(prog_name)

        parser.add_argument('name', help='Dynamic action name')
        parser.add_argument('class_name', help='Dynamic action class name')
        parser.add_argument('code_source', help='Code source ID or name')
        parser.add_argument(
            '--public',
            action='store_true',
            help='With this flag an action will be marked as "public".'
        )
        parser.add_argument(
            '--namespace',
            nargs='?',
            default='',
            help="Namespace to create the action within.",
        )

        return parser

    def take_action(self, parsed_args):
        scope = 'public' if parsed_args.public else 'private'

        mistral_client = self.app.client_manager.workflow_engine

        dyn_action = mistral_client.dynamic_actions.create(
            parsed_args.name,
            parsed_args.class_name,
            parsed_args.code_source,
            namespace=parsed_args.namespace,
            scope=scope
        )

        return DynamicActionFormatter.format(dyn_action)


class Delete(command.Command):
    """Delete action."""

    def get_parser(self, prog_name):
        parser = super(Delete, self).get_parser(prog_name)

        parser.add_argument(
            'identifier',
            nargs='+',
            help="Dynamic action name or ID (can be repeated multiple times)."
        )
        parser.add_argument(
            '--namespace',
            nargs='?',
            default='',
            help="Namespace of the dynamic action(s).",
        )

        return parser

    def take_action(self, parsed_args):
        mistral_client = self.app.client_manager.workflow_engine

        utils.do_action_on_many(
            lambda s: mistral_client.dynamic_actions.delete(
                s,
                namespace=parsed_args.namespace
            ),
            parsed_args.identifier,
            "Request to delete dynamic action(s) %s has been accepted.",
            "Unable to delete the specified dynamic action(s)."
        )


class Update(command.ShowOne):
    """Update dynamic action."""

    def get_parser(self, prog_name):
        parser = super(Update, self).get_parser(prog_name)

        parser.add_argument(
            'identifier',
            help='Dynamic action identifier (ID or name)'
        )
        parser.add_argument(
            '--class-name',
            dest='class_name',
            nargs='?',
            help='Dynamic action class name.'
        )
        parser.add_argument(
            '--code-source',
            dest='code_source',
            nargs='?',
            help='Code source identifier (ID or name).'
        )
        parser.add_argument(
            '--public',
            action='store_true',
            help='With this flag action will be marked as "public".'
        )
        parser.add_argument(
            '--namespace',
            nargs='?',
            default='',
            help="Namespace of the action.",
        )

        return parser

    def take_action(self, parsed_args):
        scope = 'public' if parsed_args.public else 'private'

        mistral_client = self.app.client_manager.workflow_engine

        dyn_action = mistral_client.dynamic_actions.update(
            parsed_args.identifier,
            class_name=parsed_args.class_name,
            code_source=parsed_args.code_source,
            scope=scope,
            namespace=parsed_args.namespace
        )

        return DynamicActionFormatter.format(dyn_action)



# Copyright 2014 - Mirantis, Inc.
# Copyright 2020 Nokia Software.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from mistralclient.api import base


class Action(base.Resource):
    resource_name = 'Action'


class ActionManager(base.ResourceManager):
    resource_class = Action

    def create(self, definition, scope='private', namespace=''):
        self._ensure_not_empty(definition=definition)

        # If the specified definition is actually a file, read in the
        # definition file
        definition = self.get_contents_if_file(definition)
        url = '/actions?scope=%s' % scope

        if namespace:
            url += '&namespace=%s' % namespace

        return self._create(
            url,
            definition,
            response_key='actions',
            dump_json=False,
            headers={'content-type': 'text/plain'},
            is_iter_resp=True
        )

    def update(self, definition, scope='private', id=None, namespace=''):
        self._ensure_not_empty(definition=definition)

        params = '?scope=%s' % scope

        if namespace:
            params += '&namespace=%s' % namespace

        url = ('/actions/%s' % id if id else '/actions') + params

        # If the specified definition is actually a file, read in the
        # definition file
        definition = self.get_contents_if_file(definition)

        return self._update(
            url,
            definition,
            response_key='actions',
            dump_json=False,
            headers={'content-type': 'text/plain'},
            is_iter_resp=True
        )

    def list(self, marker='', limit=None, sort_keys='', sort_dirs='',
             fields='', namespace='', **filters):
        if namespace:
            filters['namespace'] = namespace

        query_string = self._build_query_params(
            marker=marker,
            limit=limit,
            sort_keys=sort_keys,
            sort_dirs=sort_dirs,
            fields=fields,
            filters=filters
        )

        return self._list(
            '/actions%s' % query_string,
            response_key='actions',
        )

    def get(self, identifier, namespace=''):
        self._ensure_not_empty(identifier=identifier)

        return self._get('/actions/%s/%s' % (identifier, namespace))

    def delete(self, identifier, namespace=''):
        self._ensure_not_empty(identifier=identifier)

        self._delete('/actions/%s/%s' % (identifier, namespace))

    def validate(self, definition):
        self._ensure_not_empty(definition=definition)

        # If the specified definition is actually a file, read in the
        # definition file
        definition = self.get_contents_if_file(definition)

        return self._validate(
            '/actions/validate',
            definition,
            dump_json=False,
            headers={'content-type': 'text/plain'}
        )



# Copyright 2015 - StackStorm, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import copy
import datetime
import os
import tempfile
from unittest import mock

from oslo_serialization import jsonutils

import yaml

from mistralclient.api.v2 import environments
from mistralclient.commands.v2 import environments as environment_cmd
from mistralclient.tests.unit import base


ENVIRONMENT_DICT = {
    'name': 'env1',
    'description': 'Test Environment #1',
    'scope': 'private',
    'variables': {
        'server': 'localhost',
        'database': 'test',
        'timeout': 600,
        'verbose': True
    },
    'created_at': str(datetime.datetime.utcnow()),
    'updated_at': str(datetime.datetime.utcnow())
}

ENVIRONMENT = environments.Environment(mock, ENVIRONMENT_DICT)
EXPECTED_RESULT = (ENVIRONMENT_DICT['name'],
                   ENVIRONMENT_DICT['description'],
                   jsonutils.dumps(ENVIRONMENT_DICT['variables'], indent=4),
                   ENVIRONMENT_DICT['scope'],
                   ENVIRONMENT_DICT['created_at'],
                   ENVIRONMENT_DICT['updated_at'])

EXPECTED_EXPORT_RESULT = (ENVIRONMENT_DICT['name'],
                          ENVIRONMENT_DICT['description'],
                          ENVIRONMENT_DICT['scope'],
                          jsonutils.dumps(ENVIRONMENT_DICT['variables']))


class TestCLIEnvironmentsV2(base.BaseCommandTest):

    def _test_create(self, content):
        self.client.environments.create.return_value = ENVIRONMENT

        with tempfile.NamedTemporaryFile() as f:
            f.write(content.encode('utf-8'))
            f.flush()
            file_path = os.path.abspath(f.name)
            result = self.call(environment_cmd.Create, app_args=[file_path])
            self.assertEqual(EXPECTED_RESULT, result[1])

    def test_create_from_json(self):
        self._test_create(jsonutils.dumps(ENVIRONMENT_DICT, indent=4))

    def test_create_from_yaml(self):
        yml = yaml.dump(ENVIRONMENT_DICT, default_flow_style=False)
        self._test_create(yml)

    def _test_update(self, content):
        self.client.environments.update.return_value = ENVIRONMENT

        with tempfile.NamedTemporaryFile() as f:
            f.write(content.encode('utf-8'))
            f.flush()
            file_path = os.path.abspath(f.name)
            result = self.call(environment_cmd.Update, app_args=[file_path])
            self.assertEqual(EXPECTED_RESULT, result[1])

    def test_update_from_json(self):
        env = copy.deepcopy(ENVIRONMENT_DICT)
        del env['created_at']
        del env['updated_at']
        self._test_update(jsonutils.dumps(env, indent=4))

    def test_update_from_yaml(self):
        env = copy.deepcopy(ENVIRONMENT_DICT)
        del env['created_at']
        del env['updated_at']
        yml = yaml.dump(env, default_flow_style=False)
        self._test_update(yml)

    def test_list(self):
        self.client.environments.list.return_value = [ENVIRONMENT]
        expected = (ENVIRONMENT_DICT['name'],
                    ENVIRONMENT_DICT['description'],
                    ENVIRONMENT_DICT['scope'],
                    ENVIRONMENT_DICT['created_at'],
                    ENVIRONMENT_DICT['updated_at'])

        result = self.call(environment_cmd.List)

        self.assertListEqual([expected], result[1])

    def test_get(self):
        self.client.environments.get.return_value = ENVIRONMENT

        result = self.call(environment_cmd.Get, app_args=['name'])

        self.assertEqual(EXPECTED_RESULT, result[1])

    def test_get_with_export(self):
        self.client.environments.get.return_value = ENVIRONMENT

        result = self.call(environment_cmd.Get, app_args=['--export', 'name'])

        self.assertEqual(EXPECTED_EXPORT_RESULT, result[1])

    def test_delete(self):
        self.call(environment_cmd.Delete, app_args=['name'])

        self.client.environments.delete.assert_called_once_with('name')

    def test_delete_with_multi_names(self):
        self.call(environment_cmd.Delete, app_args=['name1', 'name2'])

        self.assertEqual(2, self.client.environments.delete.call_count)
        self.assertEqual(
            [mock.call('name1'), mock.call('name2')],
            self.client.environments.delete.call_args_list
        )
